/*
The MIT License (MIT)

Copyright (c) 2014 CantTouchDis <bauschp@informatik.uni-freiburg.de>
Copyright (c) 2014 brio1009 <christoph1009@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

#define GLM_FORCE_RADIANS

#include "parser/SceneFileParser.h"

#include <glm/glm.hpp>
#include <glm/gtc/constants.hpp>
#include <glm/gtc/matrix_transform.hpp>

#include <genfactory/GenericFactory.h>
#include <genfactory/Property.h>

#include <cstdarg>
#include <cstdio>
#include <cstring>
#include <map>
#include <string>

#include "./Scene.h"
#include "shapes/Shape.h"
// REMOVE ME!!!!
#include "cameras/PerspectiveCamera.h"
#include "materials/ColorMaterial.h"
// UP TO HERE!!!

using std::string;

struct cstringcomp {
  bool operator()(const char* s1, const char* s2) {
    return strcmp(s1, s2) < 0;
  }
};
// _____________________________________________________________________________
template<>
void SceneFileParser::parseGroupSpecial<Material>(
      rapidxml::xml_node<>* node, ...) const {
  // Get the material map from the variadic
  va_list ap;
  // Read needed stuff.
  va_start(ap, node);
  std::map<char*, Material*, cstringcomp>* materialMap =
        reinterpret_cast<std::map<
              char*,
              Material*,
              cstringcomp>*>(va_arg(ap, void*));
  va_end(ap);
  rapidxml::xml_node<>* child = node;
  while (child) {
    // add this child.
    // TODO(bauschp, Fr 8. Aug 23:28:39 CEST 2014): check if pointer alreadt ex.
    Material* mat = genfactory::GenericFactory<Material>::create(child->name());
    if (!mat) {
      child = child->next_sibling();
      continue;
    }
    // call all the needed atributes.
    for (rapidxml::xml_attribute<>* attr = child->first_attribute();
         attr; attr = attr->next_attribute()) {
      if (strcmp(attr->name(), "id") == 0) {
        materialMap->emplace(attr->value(), mat);
        continue;
      }
      mat->setFromString(attr->name(), attr->value());
    }
    child = child->next_sibling();
  }
  printf("Map now contains %zu elements.\n", materialMap->size());
}

// _____________________________________________________________________________
template<>
void SceneFileParser::parseGroupSpecial<Shape>(
      rapidxml::xml_node<>* node, ...) const {
  // Get the material map from the variadic
  va_list ap;
  // Read needed stuff.
  va_start(ap, node);
  std::map<char*, Material*, cstringcomp>* materialMap =
        reinterpret_cast<std::map<
              char*,
              Material*,
              cstringcomp>*>(va_arg(ap, void*));
  Scene* scene = reinterpret_cast<Scene*>(va_arg(ap, void*));
  va_end(ap);

  // start working.
  rapidxml::xml_node<>* child = node;
  while (child) {
    // add this child.
    rapidxml::xml_attribute<>* tmpAttr = child->first_attribute("Light");
    Shape* shape;
    if (tmpAttr && strcmp(tmpAttr->value(), "1") == 0) {
      shape = genfactory::GenericFactory<Shape>::create(std::string(child->name()) + "Important");
      shape->setMaterialPtr(new ColorMaterial(Color(1, 1, 1)));
    } else {
      shape = genfactory::GenericFactory<Shape>::create(child->name());
    }
    if (!shape) {
      child = child->next_sibling();
      continue;
    }
    // call all the needed atributes.
    for (rapidxml::xml_attribute<>* attr = child->first_attribute();
         attr; attr = attr->next_attribute()) {
      if (strcmp(attr->name(), Material::name) != 0) {
        shape->setFromString(attr->name(), attr->value());
        continue;
      }
      auto map_entry = materialMap->find(attr->value());
      if (map_entry == materialMap->end()) {
        perror("Material not defined above\n");
        continue;
      }
      shape->setFromString(
          Material::name,
          genfactory::StringCastHelper<const Material*>::toString(map_entry->second));
    }
    scene->addShape(shape);
    child = child->next_sibling();
  }
}

// _____________________________________________________________________________
void SceneFileParser::parse(const std::string& filename, Scene* scene) const {
  rapidxml::xml_document<> doc;    // character type defaults to char
  char* text = getFileContents(filename);
  if (!text)
    return;
  try {
    doc.parse<0>(text);    // 0 means default parse flags
  } catch (rapidxml::parse_error e) {
    printf("%s\n", e.what());
    printf("Problem: %s\n", (e.where<char>() - 5));
  }
  // now create the scene.
  // first of all create the materials in the material group.
  std::map<char*, Material*, cstringcomp> materialMap;
  rapidxml::xml_node<>* groupStart = parseGroup<Material>(&doc);
  parseGroupSpecial<Material>(groupStart, &materialMap);

  // now add the shapes.
  groupStart = parseGroup<Shape>(&doc);
  parseGroupSpecial<Shape>(groupStart, &materialMap, scene);
  // TODO(bauschp, Sa 9. Aug 09:08:20 CEST 2014): Read the cams from the file.
  float angle = glm::pi<float>() * 2.0f;
  glm::mat4 trans = glm::rotate(glm::mat4(1.0),
      (angle / 20) * (-1), glm::vec3(0, 1, 0));
  trans = glm::translate(trans, glm::vec3(0, 0, 23));
  PerspectiveCamera* cam = new PerspectiveCamera(1280, 720,
                                                 glm::radians(85.0f));
  cam->transform(trans);
  cam->setUsePostProcessing(false);
  scene->cameras().push_back(cam);
  // Don`t forget to free the memory!
  free(text);
}
// _____________________________________________________________________________
char* SceneFileParser::getFileContents(const std::string& filename) {
  FILE* file = fopen(filename.c_str(), "r");
  if (!file)
    return nullptr;
  // file exists.
  // get the number of bytes.
  fseek(file, 0, SEEK_END);
  // TODO(bauschp, Fr 8. Aug 22:39:05 CEST 2014): DONT TRY TO LOAD BINARY FILES!
  size_t sizeInBytes = ftell(file);
  fseek(file, 0, SEEK_SET);
  char* buffer = static_cast<char*>(malloc(sizeInBytes + 1));  // + 1 needed?
  if (fread(buffer, 1LU, sizeInBytes, file) != sizeInBytes) {
    perror("unexpected file length\n");
    fclose(file);
    free(buffer);
    return nullptr;
  }
  buffer[sizeInBytes] = 0;
  // close the file.
  fclose(file);
  return buffer;
}
